/*******************************************************************************
 Projektname:       LineRacer.cprj
 Bentigte Libss:  IntFunc_lib.cc
 Routinen:          LineRacer.cc, PRO-BOT128C_Lib.cc
 Autor:             Niko
 Datum:             28. November 2012
 Funktion:          PRO-BOT128 folgt einer schwarzen Linie auf den Boden.

                    Der PRO-BOT128 folgt einer Linie wie eine Wackelente.
                    Die Forward-LEDs zeigen an, wenn die Linie verloren wurde.
                    Die Backward-LEDs zeigen an, wenn die Sensoren die schwarze Linie
                    erkennen.

                    Die Werte BLACK und WHITE wurden durch Tests ermittelt und
                    mssen fr jeden einzelnen PRO-BOT erneut ermittelt werden.

                    Der schwarze Strich sollte mindestens 20 mm breit sein,
                    damit der PRO-BOT ihn findet und nicht verliert.
                    Auf geraden Strecken beschleunigt der PRO-BOT auf "HIGHSPEED".

                    Es werden 4 Zustnde ausgewertet:
                    Beide Sensoren auf BLACK - Geradeausfahrt mit HIGHSPEED
                    Linker Sensor auf BLACK - Linksskurve
                    Rechter Sensor auf BLACK - Rechtskurve
                    Beide Sensoren auf WHITE - HALT und kurze Strecke rckwrts

                    Das kurze Rckwrtsfahren fhrt dazu, dass der PRO-BOT meist
                    die Linie wiederfindet. Allerdings hat dieser simple Algo-
                    rithmus seine Grenzen!
 */

#define WHITE 200
#define BLACK 100
#define HIGHSPEED 200
#define MIDDLESPEED 180
#define LOWSPEED 0

int Line_left_value; // Messwert linker Liniensensor
int Line_right_value; // Messwert rechter Liniensensor


void main(void)
{
    PRO_BOT128_INIT(); //System-Setup - immer an 1. Stelle in der main
    Line_left_value = 0;
    Line_right_value = 0;
    LINE_LED_ON(); // Licht an
    DRIVE_ON();//ENABLE for Motore
    MOTOR_DIR(1,1); //Richtung "Vorwrts"


    //Main programm, endless Loop
    do
     {
     Line_left_value = READ_LINE_LEFT();
     Line_right_value = READ_LINE_RIGHT();

     // Beide Liniensensoren erkennen BLACK
      if (  (Line_left_value < BLACK) && (Line_right_value < BLACK))
     {
        DRIVE_ON();
        MOTOR_POWER(HIGHSPEED, HIGHSPEED);
        BLL_ON();
        BLR_ON();
     }
     // Nur der linke Liniensensoren erkennt BLACK
      if (  (Line_left_value < BLACK) && (Line_right_value > WHITE))
     {
        DRIVE_ON();
        MOTOR_POWER(MIDDLESPEED, LOWSPEED);
        BLL_ON();
        BLR_OFF();
     }
     // Nur der rechte Liniensensoren erkennt BLACK
      if (  (Line_left_value > WHITE) && (Line_right_value < BLACK))
     {
        DRIVE_ON();
        MOTOR_POWER(LOWSPEED, MIDDLESPEED);
        BLL_OFF();
        BLR_ON();
     }
     // Beide Liniensensoren erkennen WHITE - Linie verloren!
     if (  (Line_left_value > WHITE) && (Line_right_value > WHITE))
     {
        DRIVE_OFF();
        FLL_ON();
        FLR_ON();
        BLL_OFF();
        BLR_OFF();
        DELAY_MS(500);
        DRIVE_ON();
        MOTOR_DIR(0,0); //Richtung auf "Rckwrts", um Linie wieder zu finden
        MOTOR_POWER (MIDDLESPEED, MIDDLESPEED);
        DELAY_MS(500);
        DRIVE_OFF();
        DELAY_MS(200);
        DRIVE_ON();
        MOTOR_DIR(1,1); //Richtung wieder auf "VORWRTS"
     }
     else
     {
        FLL_OFF();
        FLR_OFF();
     }

     /*
     Mit diesem Programmteil kann man die Messwerte auf dem PC ausgeben:
     Msg_WriteInt(Line_left_value);
     Msg_WriteText("-left");
     Msg_WriteChar(13);

     Msg_WriteInt(Line_right_value);
     Msg_WriteText("-right");
     Msg_WriteChar(13);
     */

     }while (1);

}

